#=======================================================================================================
#PaREn Translator Plugins
#
#In this module you find functions for registering Import and Export PaREnData Plugins.
#This module has been separated out from the ExCoLib and is an independent module now. References
#to PaREnUtility and PaREnStatus have been removed.
#Instead if logging errors the module now crashes by exceptions.
#=======================================================================================================


#=======================================================================================================
#INCLUDES
#=======================================================================================================
from 	xml.dom.minidom import getDOMImplementation
from 	numpy import *
from 	scipy import *
from 	time import *	#ADDED DUE TO SEPARATION

import 	xml.dom.minidom
import 	csv

#Originally
#from PaREnStatus import *
#from PaREnUtility import *

#=======================================================================================================
#DEFAULTS
#=======================================================================================================
__author__      = "Atif Mehdi, Aleksander Lodwich"
__copyright__  	= "Copyright (C) 2008 DFKI"
__version__     = "0.1"

ModuleVerbosity = 1;



#=======================================================================================================
#CLASS TranslatorPlugins
#This class is responsible for handling all the formats registered by the programmer. It maintains the 
#record of import-export plugins
#=======================================================================================================
class TranslatorPlugins :
	#===================================================================================================
	#Class TranslatorPlugins 
	#CONSTRUCTOR
	#===================================================================================================
	def __init__( self ):
		self.MyRegistered = []
	#end __init__
	
	#===================================================================================================
	#Class TranslatorPlugins 
	#DESTRUCTOR
	#===================================================================================================
	def __del__( self ) :
		self.MyRegistered = []
	#end __del__
	
	#===================================================================================================
	#Class TranslatorPlugins 
	#Add()
	#Search if the plugin is already registered or not. If it is already registered, it will generate an
	#error. Otherwise will add the plugin in self.MyRegistered
	#Parameter			: PluginInfo	(Contains information about a plugin)
	#===================================================================================================
		
	def Add( self, PluginInfo ):
		if self.MyRegistered.count( PluginInfo ) == 0 :
			self.MyRegistered += [ PluginInfo ]
		else :
			#ORIGINAL
			#WrnMsg = { "Status":'Error', "Description":'Translator Plugin already exists!', "MODULE":'PaREnTranslatorPlugins::TranslatorPlugins::Add()' }
			#Send( WrnMsg, ModuleVerbosity )
			#REPLACED BY
			raise Exception, "Translator plugin already exists! (" + str( PlugInfo ) + ")"
	#end Add
	
	#===================================================================================================
	#Class TranslatorPlugins 
	#Remove()
	#Removes a plugin from self.MyRegistered.
	#Parameter			: PluginInfo	(Contains information about a plugin)
	#===================================================================================================
	def Remove( self, PluginInfo ):
		try:
			self.MyRegistered.pop( self.MyRegistered.index( PluginInfo ) )
		except:
			pass
	#end Remove

	#===================================================================================================
	#Class TranslatorPlugins 
	#Import()
	#Imports data using a given stream (Stream) from a particular format (Name). If the format is not
	#registered it generates an error. If the format is registered but the import is not supported a
	#warning is generated by class Plugin.
	#Parameter			: Name	(Name of the format importer)
	#					: Stream (File name or an object pointing to a file)
	#===================================================================================================
	def Import( self, Name, Stream, Options={} ):
		Object = None
		for Object in self.MyRegistered :
			if Object.Name().upper() == Name.upper() and Object.CanImport() > 0:
				#try:
				return Object.Import( Stream, Options )
				#except:
				#	print Object.Name(), Name, Stream, Options
				#	exit()
			#end if
		#end for
	
		#ORIGINAL
		#WrnMsg = { "Status":'Error', "Description":'Oops, Translator Plugin Not Found!', "MODULE":'PaREnTranslatorPlugins::TranslatorPlugins::Import()' }
		#Send( WrnMsg, ModuleVerbosity )
		#REPLACED BY
		print "Oops! No Translator plugin found!"
		
		return None
	#end Import

	#===================================================================================================
	#Class TranslatorPlugins 
	#Export()
	#Exports data (Data) using a given stream (Stream) to a particular format (Name). If the format is not
	#registered it generates an error. If the format is registered but the export is not supported a
	#warning is generated by class Plugin.
	#Parameter			: Name	(Name of the format importer)
	#					: Stream (File name or an object pointing to a file)
	#					: Data (Data to be exported to file)
	#===================================================================================================
	def Export( self, Name, Stream, Data, Options={} ):
		Object = None
		
		if len( Data ) > 1:
			MustHaveModes = [ Plugin.MultiCompleteData, Plugin.MultiRestrictedData ];
		else:
			MustHaveModes = [ Plugin.MultiCompleteData, Plugin.MultiRestrictedData, Plugin.SingleCompleteData, Plugin.SingleRestrictedData ];
			
		for Object in self.MyRegistered :
			if Object.Name().upper() == Name.upper() and Object.CanExport() in MustHaveModes:
				return Object.Export( Stream, Data, Options )
			#end if
		#end for
		
		#ORIGINAL
		#WrnMsg = { "Status":'Error', "Description":'Oops, Translator Plugin Not Found!', "MODULE":'PaREnTranslatorPlugins::TranslatorPlugins::Export()' }
		#Send( WrnMsg, ModuleVerbosity )
		#REPLACED BY
		print "Oops! No Translator plugin found or the right translator doesn't have the right capability!"
		return None
	#end Export
	
	#===================================================================================================
	#Class TranslatorPlugins 
	#PrintAllFormatsInfo()
	#Prints the name and availability of Import and Export Capabilities of a format
	#===================================================================================================
	def PrintAllFormatsInfo( self ):
		Object = None
		Importing = { 0 : '[No Import]', 1 : '[FullMultiSet Import]', 2 : '[FullSingleSet Import]', 3 : '[RestricedMultiSet Import]', 4 : '[RestrictedSingleSet Import]' };
		Exporting = { 0 : '[No Export]', 1 : '[FullMultiSet Export]', 2 : '[FullSingleSet Export]', 3 : '[RestricedMultiSet Export]', 4 : '[RestrictedSingleSet Export]' };
		
		print "Names of Registered plugins:"
		for Object in self.MyRegistered :
			print Object.Name(), ":", 

			print Importing[Object.CanImport()],
			print Exporting[Object.CanExport()],
			print ""
			
			if len( Object.ImportOptions() ) > 0:
				print "Import Options:", Object.ImportOptions().keys();
				
			if len( Object.ExportOptions() ) > 0:
				print "Export Options:", Object.ExportOptions().keys();
				
		#end for
	#end PrintAllFormatsInfo

#=======================================================================================================
#END OF CLASS TranslatorPlugins
#=======================================================================================================
	
#=======================================================================================================
#GLOBALS
#=======================================================================================================
TranslatorPluginRegister = TranslatorPlugins()












#=======================================================================================================
#CLASS Plugins
#It is the Base class for all the format classes.
#=======================================================================================================
class Plugin :
	
	#the following codes are to be used for import and export direction:
	NoCapability 			= 0;	#can't do anything for the direction
	MultiCompleteData		= 1;	#can read (write) multiple data sets with all data fields (like XML)
	SingleCompleteData		= 2;	#can read (write) just a single data set with all data fields (like MAT)
	MultiRestrictedData		= 3;    #can read (write) multiple data sets, but not all fields
	SingleRestrictedData	= 4;	#can read (write) only a single data set and not with all fields (like SVM)	
	
	#===================================================================================================
	#Class Plugin
	#CONSTRUCTOR
	#Registers the plugin
	#Parameter		:	Name	(Name of the format, string)
	#				:	CanImport	(Import Functionality available, 1 = Available, 0 = Not Available)
	#				:	CanExport	(Export Functionality available, 1 = Available, 0 = Not Available)
	#===================================================================================================
	def __init__( self, Name, CanImport, CanExport ):
		self.MyName = Name
		self.MyCanImport = CanImport
		self.MyCanExport = CanExport
		global TranslatorPluginRegister
		TranslatorPluginRegister.Add( self )
	#end __init__

	#===================================================================================================
	#Class Plugin
	#DESTRUCTOR
	#Because of memory handling problems of Python and its GC and just a bunch of others do not 
	#use __del__ in any python code. Plugins will never get freed up anyway.
	#===================================================================================================
	#def __del__( self ):
	#	global TranslatorPluginRegister
	#	TranslatorPluginRegister.Remove( self )
	#end __del__

	#===================================================================================================
	#Class Plugin
	#Name()
	#Returns	:	self.MyName
	#===================================================================================================
	def Name( self ):
		return self.MyName
	#end Name

	#===================================================================================================
	#Class Plugin
	#CanImport()
	#Returns	:	self.MyCanImport
	#===================================================================================================
	def CanImport( self ):
		return self.MyCanImport
	#end CanImport

	#===================================================================================================
	#Class Plugin
	#CanExport()
	#Returns	:	self.MyCanExport
	#===================================================================================================
	def CanExport( self ):
		return self.MyCanExport
	#end CanExport

	#===================================================================================================
	#Class Plugin
	#Import()
	#It will be called only when the derived class does not have the capability to Import the format. It
	#will generate a warning to notify user about unavailability of Import Function.
	#Returns	:	None
	#===================================================================================================
	def Import( self, Stream, Options={} ):
		#ORIGINAL
		#WrnMsg = { "Status":'Warning', "Description":'Oops, was not implemented!', "MODULE":'PaREnTranslatorPlugins::Plugin::Import()' }
		#Send( WrnMsg, ModuleVerbosity )
		#REPLACED BY
		print "Import() method not implemented!";
		return None;
	#end Import

	#===================================================================================================
	#Class Plugin
	#Export()
	#It will be called only when the derived class does not have the capability to Export the format. It
	#will generate a warning to notify user about unavailability of Export Function.
	#Returns	:	None
	#===================================================================================================
	def Export( self, Stream, Data, Options={} ):
		#ORIGINAL
		#WrnMsg = { "Status":'Warning', "Description":'Oops, was not implemented!', "MODULE":'PaREnTranslatorPlugins::Plugin::Export()' }
		#Send( WrnMsg, ModuleVerbosity )
		#REPLACED BY
		print "Export() method not implemented!";
		return None;
	#end Export


	#===================================================================================================
	#Class Plugin
	#ImportOptions()
	#RETURNS DICT
	#Returns a dictionary with default options for importing
	#===================================================================================================
	def ImportOptions( self ):
		return {};
	
	#===================================================================================================
	#Class Plugin
	#ExportOptions()
	#RETURNS DICT
	#Returns a dictionary with default options for exporting
	#===================================================================================================
	def ExportOptions( self ):
		return {};
	
#=======================================================================================================
#END OF CLASS Plugin
#=======================================================================================================
	

#=======================================================================================================
#CLASS RPlugin
#=======================================================================================================
class RPlugin( Plugin ):
	
	#===================================================================================================
	#CLASS RPlugin
	#CONSTRUCTOR
	#===================================================================================================
	def __init__( self ):
		MyCanImport = Plugin.NoCapability;
		MyCanExport = Plugin.MultiCompleteData;
		Plugin.__init__( self, 'R', MyCanImport, MyCanExport );

	#===================================================================================================
	#CLASS RPlugin
	#FUNCTION Export( Stream, Data, Options )
	#RETURNS 1/0
	#Exports data into an R script. Invoke source( 'filename.R' ) in order to execute it.
	#Call "Imported" on the command line of R in order to find out which variables got imported.
	#Stream		: FILE or other STREAM. Target file for R content.
	#Data		: LIST OF DICTS. List of dictionaries containing the data from a translator. (or CreateData())
	#Options	: DICT. Eventual options for the export functions. This one doesn't have any.
	#===================================================================================================
	def Export( self, Stream, Data, Options={} ):
		i = 0;
		ExportedItems = "IMPORTED ITEMS ARE: ";
		for Set in Data:
			i += 1;
			if Set.has_key( "Vectors" ):
				RefVectors = Set.get( "Vectors", array([[]]) );
				Dims = range( RefVectors.shape[1] );
				Vecs = range( RefVectors.shape[0] );
				Features = [];
				
				if Set.get( "Features", None ) is None:
					for d in Dims:
						Features += [ 'dim' + str( d ) ]; #make sure there are resonable names
				else: 
					for d in Dims:
						Features += [ 'dim' + str( Set["Features"][d] ) ]; #or use original names, but prepend dim_, because in python classids can be integers
				
				#in R you must specify your data by column. There is no way to specify it by row.
				#so here we go and extract columns of data:
				for d in Dims: 
					Stream.write( Features[d] + ' <- c( ' );
					for v in Vecs:
						Stream.write( str( RefVectors[v,d] ) );
						if v < len( Vecs ) - 1:
							Stream.write( ", " );
					Stream.write( " );\n" );
					
				Stream.write( "Vectors" + str(i) + "<- data.frame( " );
				for d in Dims:
					Stream.write( Features[d] );
					if d < len( Dims ) - 1:
						Stream.write( ", " );
				Stream.write( " );\n" );
				
				ExportedItems += "Vectors" + str(i) + " ";
			#end if has Vectors
			
			#write a R column with 
			if Set.has_key( "Labels" ):				
				Stream.write( "Labels" + str( i ) + "<- c( " );
				Count = len( Set["Labels"] );
				for l in range( Count ):
					Stream.write( "'Class_" + str( Set["Labels"][l] ) + "'" );
					if l < Count - 1:
						Stream.write( ", " );
				Stream.write( " );\n" );
				ExportedItems += "Labels" + str(i) + " ";
				
			#If Probabilities are available, then also export those...
			if Set.has_key( "Probabilities" ):
				RefVectors = Set.get( "Probabilities", array([[]]) );
				Dims = range( RefVectors.shape[1] );
				Vecs = range( RefVectors.shape[0] );
				Features = [];
				
				if Set.get( "ClassIDs", None ) is None:
					for d in Dims:
						Features += [ 'Class' + str( d ) ];
				else: 
					for d in Dims:
						Features += [ 'Class_' + str( Set["ClassIDs"][d] ) ];
				
				#in R you must specify your data by column. There is no way to specify it by row.
				#so here we go and extract columns of data:
				for d in Dims:
					Stream.write( Features[d] + ' <- c( ' );
					for v in Vecs:
						Stream.write( str( RefVectors[v,d] ) );
						if v < len( Vecs ) - 1:
							Stream.write( ", " );
					Stream.write( " );\n" );
					
				Stream.write( "Probabilities" + str(i) + "<- data.frame( " );
				for d in Dims:
					Stream.write( Features[d] );
					if d < len( Dims ) - 1:
						Stream.write( ", " );
				Stream.write( " );\n" );
				
				ExportedItems += "Probabilities" + str(i) + " ";
			#end if has Probabilities
			
		#end for every Set in Data
		
		Stream.write( 'Imported <- "' + ExportedItems + '";\n' );
		return 1;
	#end def Export to R
#=======================================================================================================
#CLASS MATPlugin
#A MAT Plugin class. Supports Import and Export from and the MATLAB format.
#=======================================================================================================
class MATPlugin( Plugin ):
	
	#===================================================================================================
	#Class MATPlugin
	#CONSTRUCTOR
	#===================================================================================================	
	def __init__( self ):
		MyCanImport = Plugin.SingleCompleteData;
		MyCanExport = Plugin.SingleCompleteData;
		Plugin.__init__( self, 'MAT', MyCanImport, MyCanExport );
		
	#===================================================================================================
	#Class MATPlugin
	#Import()
	#Imports data from a MAT file and returns an array of dictionary items having Vectors, Labels and 
	#Probabilities. A dictionary entry will be 'None' if corresponding Data is not available.
	#Stream				:	Stream	(filename or an object of an opened file)
	#Options			:	Dictionary with optional values
	#===================================================================================================	
	def Import( self, Stream, Options={} ):
		X = io.loadmat( Stream );
		ValidKeys = [ "Name", "Vectors", "Labels", "ClassIDs", "Probabilities" ];
		
		R = {}
		for Key in ValidKeys:
			R[Key] = X.get( Key, None );
			
		return R;
	
	#===================================================================================================
	#Class MATPlugin
	#Export()
	#RETURNS 1/0 (SUCCESS/FAILURE)
	#Exports data to a Matlab file. Only supports single data sets.
	#Stream				:	Stream	(filename or an object of an opened file)
	#Options			:	Dictionary with optional values
	#===================================================================================================	
	def Export( self, Stream, Data, Options={} ):
		io.savemat( Stream, Data[0] );
		return 1;
	

#=======================================================================================================
#CLASS SVMPlugin
#An SVM Plugin class. Supports only Export to svmlight format which is also used by svmlib.
#=======================================================================================================
class SVMPlugin( Plugin ):
	
	#===================================================================================================
	#Class SVMPlugin
	#CONSTRUCTOR
	#===================================================================================================	
	def __init__( self ):
		MyCanImport = Plugin.NoCapability;
		MyCanExport = Plugin.SingleRestrictedData;
		Plugin.__init__( self, 'SVM', MyCanImport, MyCanExport );
	
	#===================================================================================================
	#Class SVMPlugin
	#ExportOptions()
	#RETURNS DICT
	#Gets you default settings for the Export function operation.
	#===================================================================================================
	def ExportOptions( self ):
		return { "Plus":None, "Minus":None, "Zeros":1 }; #get classes automatically
	
	#===================================================================================================
	#Class SVMPlugin
	#Export()
	#RETURNS 1/0
	#Exports a single data set into a stream so that it can be read by SVMligh and SVMlib. If Plus
	#and Minus options are not supplied, then arbitrary class is taken to +1 and the rest is -1.
	#You can supply either Plus or Minus or both. If you supply Plus or Minus only, the opposite
	#class consists of the complimentary ClassIDs. If you supply both, Plus and Minus, then
	#samples classified outside those lists are exported as ZERO. If you don't want to export
	#zero labeled vectors, then set the option "Zeros" to 0.
	#Stream				:	Stream	(filename or an object of an opened file)
	#Data				: 	LIST OF DICTS. DICTS can be constructed using CreateData().
	#Options			:	Dictionary with optional values.
	#						Supported options are: Plus (LIST OF CLASSID), Minus (LIST OF CLASSID) and Zeros (1/0)
	#===================================================================================================	
	def Export( self, Stream, Data, Options={} ):		
		if len( Data ) == 0:
			return 0;
		
		#SECTION B: Identify proper action combination, this is antique code from pre-plugin-time, so don't wonder
		SAVE_VECTORS   = 'V';
		SAVE_LABELS    = 'L';
		SAVE_PROBABS   = 'P';
		HasVectors	   	= Data[0].has_key( "Vectors" );
		HasLabels      	= Data[0].has_key( "Labels" );
		ExportZeroClass	= Options.get( "Zeros", 1 );
		
		if not HasVectors:
			print "PaREnTranslatorPlugins::SVMPlugin::Export(): Vectors must be provided!";
			return 0;

		if not HasLabels:
			print "PaREnTranslatorPlugins::SVMPlugin::Export(): Labels must be provided!";
			return 0;
		
		#SECTION C: Here we deal with the creation of the Plus and Minus list. Class symbols in plus are assigned +1 and class symbols in minus aus assigned -1.
		#other samples are exported as 0.
		Plus = Options.get( "Plus", None );
		Minus = Options.get( "Minus", None );		

		#make sure Plus is a list
		if type( Plus ) is not list and type( Plus ) is not tuple and Plus is not None: 
			Plus = [Plus];
			
		#make sure Minus is a list
		if type( Minus ) is not list and type( Minus ) is not tuple and Minus is not None:
			Minus = [Minus];
		
		#inventory of availble symbols
		AllClasses = unique( Data[0]["Labels"] ).tolist()
		
		if Plus is None and Minus is None:
			Plus = [Data[0]["Labels"][0]]; #take first

		if Minus is None: #compute remaining set for Minus - this will happen in two cases, when Plus is set or Plus is None
			Minus = AllClasses;
			for Value in Plus:
				Minus.remove( Value );

		if Plus is None: #compute remainin set for Plus - this will happen in two cases, when Minus is set or Minus is None
			Plus = AllClasses;
			for Value in Minus:
				Plus.remove( Value );

		#SECTION C: Execute export configuration		
		Stream.write( "#=================================================================================\n" );
		Stream.write( "#PaREn Data Source Translator\n" );
		Stream.write( "#Stamp = " + ctime( time() ) + "\n" ); 
		Stream.write( "#=================================================================================\n" );
		
		Stream.write( "#=================================================================================\n" );
		Stream.write( "#Class Label Explaination:\n" );
		Stream.write( "#Class +1 is (" + str( Plus ) + ")\n" );
		Stream.write( "#Class -1 is (" + str( Minus ) + ")\n" );
		Stream.write( "#=================================================================================\n" );
		
		Stream.write( "#=================================================================================\n" );
		Stream.write( "#SVM Data\n" );
		Stream.write( "#=================================================================================\n" );

		VectorCount = Data[0]["Vectors"].shape[0];
		Dims 		= range( Data[0]["Vectors"].shape[1] );
		
		#print every vector
		for Row in range( VectorCount ):
			Vector = Data[0]["Vectors"][Row]; #get reference to the vector
			Label = Data[0]["Labels"][Row]; #get reference to the corresponding label
			
			WriteRest = 1;	#normally, the line is not suppressed, hence 1
			
			#identify the SVM classification, given Plus and Minus
			if Label in Plus:
				Stream.write( "+1 " );
			elif Label in Minus:
				Stream.write( "-1 " );
			elif ExportZeroClass:
				Stream.write( "0  " );
			else: WriteRest = 0; #if this is a ZERO class and it is not supposed to be exported, then change WriteRest to 0.
				
			if WriteRest: #if allowed to write...
				for d in Dims: #then check, if some values are zero. 
					if Vector[d] != 0: #if they are not zero, then write them out to stream:
						Stream.write( str( d + 1 ) + ":" + str( Vector[d] ) + " " ); 
				Stream.write( "\n" );
				
		#write tail.
		Stream.write( "#=================================================================================\n" );
		Stream.write( "#END OF FILE\n" );
		Stream.write( "#=================================================================================\n" );
		return 1;


#=======================================================================================================
#CLASS XMLPlugin
#An XML Plugin class. Supports Import and Export from and to XML format.
#=======================================================================================================
class XMLPlugin( Plugin ):
	#===================================================================================================
	#Class XMLPlugins 
	#CONSTRUCTOR
	#===================================================================================================	
	def __init__( self ):
		MyCanImport = Plugin.MultiCompleteData;
		MyCanExport = Plugin.MultiCompleteData;
		Plugin.__init__( self, 'XML', MyCanImport, MyCanExport )
	#end __init__

	#===================================================================================================
	#Class XMLPlugins 
	#Import()
	#Imports data from an XML file and returns an array of dictionary items having Vectors, Labels and 
	#Probabilities. A dictionary entry will be 'None' if corresponding Data is not available in the XML
	#Parameter			:	Stream	(filename or an object of an opened file)
	#Returns			:	An array of dictionary values having Vectors, Labels and Probabilities
	#===================================================================================================	
	def Import( self, Stream, Options={} ):
		a  = []
		#read in xml file
		try:
			doc 			= xml.dom.minidom.parse(Stream)
		except:
			#ORIGINAL
			#WrnMsg = { "Status":'Error', "Description":'Oops, File not Found, or Errors in File', "MODULE":'PaREnTranslatorPlugins::XMLPlugins::Import()' }
			#Send( WrnMsg, ModuleVerbosity )
			#REPLACED BY 
			print "File not found or file is erroneous!";
			return a
			
		#search for "PaREnData in XML File
		for e in doc.getElementsByTagName("PaREnData") :
			Data = { "Name":e.getAttribute("Name") };
			
			for x in e.childNodes :
				if x.localName == "Vectors":
					try:
						Data["Features"] = eval( x.getAttribute( 'Features' ) );
					except: pass;
					Data["Vectors"] = array( eval( x.childNodes[0].nodeValue ) )
				#end if
				
				if x.localName == "Labels" :
					Data["Labels"] = eval( x.childNodes[0].nodeValue )
				#end if
				
				if x.localName == "Probabilities" :
					Data["ClassIDs"] = eval( x.getAttribute( 'ClassIDs' ) );
					Data["Probabilities"] = array( eval( x.childNodes[0].nodeValue ) )
				#end if
				
			#end for
			a += [ Data ] #adds dictionary in the array

		#end for
		return a
	#end Import

	#===================================================================================================
	#Class XMLPlugins 
	#Export()
	#Exports data to an XML file 
	#Parameter			:	Stream	(filename or an object of an opened file)
	#					:	Data	(Data to be exported)
	#===================================================================================================	
	def Export( self, Stream, Data, Options={} ):
		DOMXML 		= getDOMImplementation();
		XMLDocument = DOMXML.createDocument( None, 'PaREnXML', None );
		Length 		= len( Data );
		
		for i in range( Length ):						
			XMLData = XMLDocument.documentElement.appendChild( XMLDocument.createElement( 'PaREnData' ) );
			if Data[i].has_key( "Name" ):
				XMLData.setAttribute( 'Name', Data[i]["Name"] )
				
			#check the dimension of Vectors
			if Data[i].has_key( "Vectors" ):
				VectorsStorage 	= XMLDocument.createElement( 'Vectors' )	
				if Data[i].has_key( "Features" ):
					VectorsStorage.setAttribute( 'Features', str( Data[i]["Features"] ) );
				VectorsData		= str( Data[i]["Vectors"].tolist() )
				VectorsStorage.appendChild( XMLDocument.createTextNode( VectorsData ) )
				XMLData.appendChild( VectorsStorage )
			#end if

			#check the dimension of Labels
			if Data[i].has_key( "Labels" ):
				LabelsStorage 	= XMLDocument.createElement( 'Labels' )
				LabelsData		= str( Data[i]["Labels"] )
				LabelsStorage.appendChild( XMLDocument.createTextNode( LabelsData ) )
				XMLData.appendChild( LabelsStorage )
			#end if
			
			#check the dimension of Probabilities
			if Data[i].has_key( "Probabilities" ):
				ProbabilitiesStorage 	= XMLDocument.createElement( 'Probabilities' )
				ProbabilitiesStorage.setAttribute( 'ClassIDs', str( Data[i]["ClassIDs"] ) );
				ProbabilitiesData		= str( Data[i]["Probabilities"].tolist() )
				ProbabilitiesStorage.appendChild( XMLDocument.createTextNode( ProbabilitiesData ) )
				XMLData.appendChild( ProbabilitiesStorage )
			#end if
		#end for
		XMLDocument.writexml( Stream )
		XMLDocument.unlink()		
		return 1;
	#end Export

#=======================================================================================================
#END OF CLASS XMLPlugin
#=======================================================================================================
		
		
#=======================================================================================================
#CLASS CSVPlugin
#An CSV Plugin class. Supports Import and Export from and to CSV format.
#=======================================================================================================
class CSVPlugin( Plugin ):
	#===================================================================================================
	#Class CSVPlugins 
	#CONSTRUCTOR
	#===================================================================================================	
	def __init__( self ):
		MyCanImport = Plugin.MultiCompleteData;
		MyCanExport = Plugin.MultiCompleteData;
		Plugin.__init__( self, 'CSV', MyCanImport, MyCanExport )
	#end __init__

	#===================================================================================================
	#Class CSVPlugins 
	#Import()
	#Imports data from an CSV file and returns an array of dictionary items having Vectors, Labels and 
	#Probabilities. A dictionary entry will be 'None' if corresponding Data is not available in the CSV
	#Parameter			:	Stream	(filename or an object of an opened file)
	#Returns			:	An array of dictionary values having Vectors, Labels and Probabilities
	#===================================================================================================	
	def Import( self, Stream, Options={} ):
		a  = []

		try:
			reader 			= csv.reader( Stream, delimiter=';' );
		except:
			#ORIGINAL
			#WrnMsg = { "Status":'Error', "Description":'Oops, File not Found, or Errors in File', "MODULE":'PaREnTranslatorPlugins::CSVPlugins::Import()' }
			#Send( WrnMsg, ModuleVerbosity )
			#REPLACED BY
			print "File not found or file is erroneous!"
			return a
		#end try
		
		#look for "PaREnData" in CSV file header
		Data = reader.next()
		Embedded = 0;
		if Data[0] == "PaREnCSV":
			Embedded = eval( Data[1] );
		
		for SetIndex in range( Embedded ):
			Data = reader.next();
			if Data[0] == "PaREnData":
				Rows = int( Data[1] );
				Vecs = int( Data[2] );
				Labs = int( Data[3] );
				Prob = int( Data[4] );

				ClassIDs = eval( Data[5] );
				Vectors = zeros( ( Rows, Vecs ) );
				Probabilities = zeros( ( Rows, Prob ) );
				
				if Labs:
					Labels = [];
				else: Labels = None;
				
				for Row in range( Rows ):
					Data = reader.next();					
					for d in range( Vecs ):
						Vectors[Row,d] = double( Data[d] );
					#end for all vec columns
					
					if Labs:
						Labels += [Data[Vecs]];
						
					for p in range( Vecs+Labs, Vecs+Labs+Prob ):
						Probabilities[Row,p-Vecs-Labs] = double( Data[p] );
					#end for all prob columns
				#end for all rows
				
				Set = {};
				if Vecs > 0:
					Set["Vectors"] = Vectors;
				if Labs:
					Set["Labels"] = Labels;
				if Prob > 0:
					Set["Probabilities"] = Probabilities;
					Set["ClassIDs"] = ClassIDs;
					
				a += [ Set ];
			else:
				raise Exception, "Format Error! Expected PaREnData!";
			#end if parendata
		#for SetIndex
		return a
	#end Import


	#===================================================================================================
	#Class CSVPlugins 
	#Export()
	#Exports data to an CSV file 
	#Parameter			:	Stream	(filename or an object of an opened file)
	#					:	Data	(Data to be exported)
	#NOTE: this function is using Stream and expects an object of an opened file
	#===================================================================================================	
	def Export( self, Stream, Data, Options={} ):
		Embedded = len( Data );
		Stream.write( "PaREnCSV;"+str(Embedded)+"\n" ); #announce the number of data sections
		
		for Set in Data:	#go through every supplied set and write it to file
			Vectors = 0;
			Labels  = 0;
			Probabilities = 0;
			HasLabels = 0;
			Dims = 0;
			ProbDims = 0;
			ClassIDs = "";
			
			if Set.has_key( "Vectors" ):
				Vectors = Set["Vectors"].shape[0];
				Dims = Set["Vectors"].shape[1];
			
			if Set.has_key( "Labels" ):
				Labels = len( Set["Labels"] );
				HasLabels = 1;
				
			if Set.has_key( "Probabilities" ):
				Probabilities = Set["Probabilities"].shape[0];
				ProbDims = Set["Probabilities"].shape[1];
				ClassIDs = str( Set["ClassIDs" ] );
				
			Rows = max( Vectors, Labels, Probabilities ); #take only the largest number of rows into account. If max values are not the same, then it will crash
			
			Stream.write( "PaREnData;" + str( Rows ) + ";" + str(Dims) + ";" + str(HasLabels) + ";" + str(ProbDims) + ";" + ClassIDs + "\n" ); #write data section header
			
			#go through every row in the data section and write the table entries:
			for i in range( Rows ):
				for d in range( Dims ):
					Stream.write( str( Set["Vectors"][i,d] ) + ";" );
					
				if HasLabels:
					Stream.write( str( Set["Labels"][i] ) + ";" );
				
				for p in range( ProbDims ):
					Stream.write( str( Set["Probabilities"][i,p] ) + ";" );
				
				Stream.write( "\n" );
		#end for
		return 1;
	#end Export


#=======================================================================================================
#END OF CLASS CSVPlugin
#=======================================================================================================



#=======================================================================================================
#SELF TEST
#NOTE: Import and Export file names should be different as it is assumed that the programmer will check
#himself, if the file exists or not and then take the appropriate action. Otherwise, Import file will be
#overwritten be Export file.
#result := [{},{},..]
#=======================================================================================================
_XML = XMLPlugin()
_CSV = CSVPlugin()
_SVM = SVMPlugin();
_MAT = MATPlugin();
_RRR = RPlugin();

#=======================================================================================================
#FUNCTION PluginInfo()
#RETURNS VOID
#Prints information about available formats on screen.
#=======================================================================================================
def PluginInfo():
	TranslatorPluginRegister.PrintAllFormatsInfo()

#=======================================================================================================
#FUNCTION CreateData( Name, Vectors, Labels, Probabilities, ClassIDs )
#RETURNS LIST OF DICTS
#This class packages the parameters into a single dictionary understood by the plugins.
#In order to store more than one translator in the file, you can do the following:
#Data = CreateData(...) + CreateData(...) + ...
#
#Name		: OPTIONAL STRING. Name of the translator
#Vectors	: OPTIONAL 2D ARRAY. Rows are feature vectors.
#Labels		: OPTIONAL LIST. Contains labels for each row
#Probabilitis:OPTIONAL 2D ARRAY. Probability of each feature vector to belong to a specific class
#ClassIDs	: MANDATORY TO PROBABILITIES. LIST. Identifies the headers (label) for each column in Probabilities.
#=======================================================================================================
def CreateData( Name=None, Vectors=None, Features=None, Labels=None, Probabilities=None, ClassIDs=None ):
	ResultStructure = {};
	if Name is not None:
		if type( Name ) is str:
			ResultStructure["Name"] = Name;
	
	if Vectors is not None:
		if type( Vectors ) is ndarray:
			ResultStructure["Vectors"] = Vectors;
	
	if Features is not None:
		if type( Features ) is list:
			ResultStructure["Features"] = Features;
	
	if Labels is not None:
		if type( Labels ) is not list:
			raise Exception, "Labels must be a list!"
		else:
			ResultStructure["Labels"] = Labels;

	if ClassIDs is not None and Probabilities is not None:
		if type(Probabilities) is ndarray and type( ClassIDs ) is list:
			ResultStructure["Probabilities"] = Probabilities;
			ResultStructure["ClassIDs"] = ClassIDs;
		
	return [ResultStructure];
		
	
#=======================================================================================================
#FUNCTION Import( FilenameWithExtension, Options={} )
#RETURNS LIST OF DICTS
#This function imports the specified file and respects the supplied options
#FilenameWithExtension	: STRING. Filename. Extension is used as type code for plug specification
#Options				: DICT. Put in options for the plugins. Options not recognized by plugin are ignored.
#=======================================================================================================
def Import( FilenameWithExtension, Options={} ):
	Extension = FilenameWithExtension[-3:].upper();
	File = open( FilenameWithExtension, "r" );
	Data = TranslatorPluginRegister.Import( Extension, File, Options );
	File.close();
	return Data;


#=======================================================================================================
#FUNCTION Export( FilenameWithExtension, Data, Options={} )
#RETURNS 1/0
#This function exports the specified data to a file and respects the supplied options
#FilenameWithExtension	: STRING. Filename. Extension is used as type code for plug specification
#Data					: LIST OF DICTS. Structure i.e. created by CreateData(). 
#Options				: DICT. Put in options for the plugins. Options not recognized by plugin are ignored.
#=======================================================================================================
def Export( FilenameWithExtension, Data, Options={} ):
	Extension = FilenameWithExtension[-3:].upper();
	File = open( FilenameWithExtension, "w" );
	Result = TranslatorPluginRegister.Export( Extension, File, Data, Options );
	File.close();
	return Result;


#=======================================================================================================
#SELF TEST
#NOTE: Import and Export file names should be different as it is assumed that the programmer will check
#himself, if the file exists or not and then take the appropriate action. Otherwise, Import file will be
#overwritten be Export file.
#result := [{},{},..]
#=======================================================================================================
if __name__ == "__main__":
	print "PaREnTranslatorPlugins (Stand Alone Version) - SELFTEST"
	PluginInfo()
	NoVecs = 100;
	NoFeat = 4;
	Labels = [ 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h' ];

	ArtificialVectors = rand( NoVecs, NoFeat );
	NoClasses = len( Labels );
	ArtificialLabels = [];
	for i in range( NoVecs ):
		ArtificialLabels += [Labels[int(rand()*NoClasses)]];
	ArtificialProbabilities = rand(NoVecs,NoClasses);
	
	#Test plugins with MULTI data capability
	Data = CreateData( "Testdata A", ArtificialVectors, None, ArtificialLabels, ArtificialProbabilities, Labels ) + CreateData( "Testdata B", Probabilities=ArtificialProbabilities, ClassIDs=Labels );

	if 1:
		Target = open( "test.xml", "w" );
		TranslatorPluginRegister.Export( "XML", Target, Data );
		Target.close();
		Source = open( "test.xml", "r" );
		RData = TranslatorPluginRegister.Import( "XML", Source );
		Source.close();

	if 1:
		Target = open( "test.csv", "w" );
		TranslatorPluginRegister.Export( "CSV", Target, Data );
		Target.close();
		Source = open( "test.csv", "r" );
		RData = TranslatorPluginRegister.Import( "CSV", Source );
		Source.close();

	if 1:
		Target = open( "test.r", "w" );
		TranslatorPluginRegister.Export( "R", Target, Data );
		Target.close();

	Export( "test.xml", Data );
	RData = Import( "test.xml" );
	Export( "test.csv", Data );
	RData = Import( "test.csv" );

	#Test modules with SINGLE data capability
	Data = CreateData( "Testdata", ArtificialVectors, None, ArtificialLabels, ArtificialProbabilities, Labels );

	if 1:
		Target = open( "test1.svm", "w" );
		TranslatorPluginRegister.Export( "SVM", Target, Data, { "Plus":"b", "Minus":"e" } );
		Target.close();
		
		Target = open( "test2.svm", "w" );
		TranslatorPluginRegister.Export( "SVM", Target, Data, { "Plus":"b" } );
		Target.close();
		
		Target = open( "test3.svm", "w" );
		TranslatorPluginRegister.Export( "SVM", Target, Data, { "Minus":"e" } );
		Target.close();
		
		Target = open( "test4.svm", "w" );
		TranslatorPluginRegister.Export( "SVM", Target, Data, { "Plus":[ "a", "b" ], "Minus":[ "c", "d", "e" ], "Zeros":0 } );
		Target.close();
		
	if 1:
		Target = open( "test.mat", "w" );
		TranslatorPluginRegister.Export( "MAT", Target, Data );
		Target.close();
		
		Target = open( "test.mat", "r" );
		RData = TranslatorPluginRegister.Import( "MAT", Target );
		Target.close();

	Export( "test.svm", Data );
	Export( "test.mat", Data );
	RData = Import( "test.mat" );


	print "SELFTEST PASSED"

#=======================================================================================================
#END OF FILE
#=======================================================================================================
